@@ -2,6 +2,7 @@
 // SPDX-License-Identifier: GPL-3.0-only
 
 #include "neochatroom.h"
+#include "chatbartype.h"
 
 #include <QFileInfo>
 #include <QMediaMetaData>
@@ -39,13 +40,11 @@
 #include <Quotient/events/simplestateevents.h>
 #include <Quotient/jobs/downloadfilejob.h>
 #include <Quotient/qt_connection_util.h>
-#if Quotient_VERSION_MINOR > 9 || (Quotient_VERSION_MINOR == 9 && Quotient_VERSION_PATCH > 1)
 #include <Quotient/thread.h>
-#endif
 
 #include "chatbarcache.h"
 #include "clipboard.h"
-#include "events/pollevent.h"
+#include "eventhandler.h"
 #include "filetransferpseudojob.h"
 #include "neochatconnection.h"
 #include "roomlastmessageprovider.h"
@@ -56,6 +55,7 @@
 #include <KIO/Job>
 #include <KIO/JobTracker>
 #endif
+
 #include <KJobTrackerInterface>
 #include <KLocalizedString>
 
@@ -131,11 +131,13 @@ NeoChatRoom::NeoChatRoom(Connection *connection, QString roomId, JoinState joinS
         this,
         [this]() {
             updatePushNotificationState(u"m.push_rules"_s);
+            loadPinnedMessage();
 
             Q_EMIT canEncryptRoomChanged();
             Q_EMIT inviteTimestampChanged();
         },
         Qt::SingleShotConnection);
+    connect(this, &Room::pinnedEventsChanged, this, &NeoChatRoom::loadPinnedMessage);
     connect(this, &Room::changed, this, [this] {
         Q_EMIT canEncryptRoomChanged();
         Q_EMIT parentIdsChanged();
@@ -260,7 +262,10 @@ QCoro::Task<void> NeoChatRoom::doUploadFile(QUrl url, QString body, std::optiona
 
         QTemporaryFile file;
         file.setFileTemplate(QStringLiteral("XXXXXX.jpg"));
-        file.open();
+        auto ok = file.open();
+        if (!ok) {
+            qWarning() << "Failed to open" << file.fileName() << file.errorString();
+        }
 
         const auto thumbnailImage = sink.videoFrame().toImage();
         Q_UNUSED(thumbnailImage.save(file.fileName()))
@@ -282,11 +287,7 @@ QCoro::Task<void> NeoChatRoom::doUploadFile(QUrl url, QString body, std::optiona
         content = new EventContent::FileContent(url, fileInfo.size(), mime, fileInfo.fileName());
     }
 
-#if Quotient_VERSION_MINOR > 9 || (Quotient_VERSION_MINOR == 9 && Quotient_VERSION_PATCH > 2)
     QString txnId = postFile(body.isEmpty() ? url.fileName() : body, std::unique_ptr<EventContent::FileContentBase>(content), relatesTo);
-#else
-    QString txnId = postFile(body.isEmpty() ? url.fileName() : body, std::unique_ptr<EventContent::FileContentBase>(content));
-#endif
     setHasFileUploading(true);
     connect(this, &Room::fileTransferCompleted, [this, txnId](const QString &id, FileSourceInfo) {
         if (id == txnId) {
@@ -387,9 +388,14 @@ const RoomEvent *NeoChatRoom::lastEvent(std::function<bool(const RoomEvent *)> f
         if (auto lastEvent = eventCast<const RoomMessageEvent>(event)) {
             return lastEvent;
         }
+
         if (auto lastEvent = eventCast<const PollStartEvent>(event)) {
             return lastEvent;
         }
+
+        if (auto lastEvent = eventCast<const EncryptedEvent>(event)) {
+            return lastEvent;
+        }
     }
 
     if (m_cachedEvent != nullptr) {
@@ -469,20 +475,19 @@ void NeoChatRoom::onRedaction(const RoomEvent &prevEvent, const RoomEvent & /*af
     }
 }
 
-QDateTime NeoChatRoom::lastActiveTime()
+QDateTime NeoChatRoom::lastActiveTime() const
 {
-    if (timelineSize() == 0) {
-        if (m_cachedEvent != nullptr) {
-            return m_cachedEvent->originTimestamp();
-        }
-        return QDateTime();
-    }
-
-    if (auto event = lastEvent()) {
+    // Find the last relevant event:
+    if (const auto event = lastEvent(m_hiddenFilter)) {
         return event->originTimestamp();
     }
 
-    // no message found, take last event
+    // If nothing is loaded yet, and there is no cached event:
+    if (timelineSize() == 0) {
+        return {};
+    }
+
+    // No message found, take last event:
     return messageEvents().rbegin()->get()->originTimestamp();
 }
 
@@ -505,12 +510,9 @@ QUrl NeoChatRoom::avatarMediaUrl() const
 
 void NeoChatRoom::changeAvatar(const QUrl &localFile)
 {
-    const auto job = connection()->uploadFile(localFile.toLocalFile());
-    if (isJobPending(job)) {
-        connect(job, &BaseJob::success, this, [this, job] {
-            connection()->callApi<SetRoomStateWithKeyJob>(id(), "m.room.avatar"_L1, QString(), QJsonObject{{"url"_L1, job->contentUri().toString()}});
-        });
-    }
+    connection()->uploadFile(localFile.toLocalFile()).onResult([this](const auto &job) {
+        connection()->callApi<SetRoomStateWithKeyJob>(id(), "m.room.avatar"_L1, QString(), QJsonObject{{"url"_L1, job->contentUri().toString()}});
+    });
 }
 
 void NeoChatRoom::toggleReaction(const QString &eventId, const QString &reaction)
@@ -560,6 +562,9 @@ bool NeoChatRoom::containsUser(const QString &userID) const
 
 bool NeoChatRoom::canSendEvent(const QString &eventType) const
 {
+    if (roomCreatorHasUltimatePowerLevel() && isCreator(localMember().id())) {
+        return true;
+    }
     auto plEvent = currentState().get<RoomPowerLevelsEvent>();
     if (!plEvent) {
         return false;
@@ -572,6 +577,9 @@ bool NeoChatRoom::canSendEvent(const QString &eventType) const
 
 bool NeoChatRoom::canSendState(const QString &eventType) const
 {
+    if (roomCreatorHasUltimatePowerLevel() && isCreator(localMember().id())) {
+        return true;
+    }
     auto plEvent = currentState().get<RoomPowerLevelsEvent>();
     if (!plEvent) {
         return false;
@@ -614,8 +622,8 @@ void NeoChatRoom::deleteMessagesByUser(const QString &user, const QString &reaso
 
 QString NeoChatRoom::historyVisibility() const
 {
-    if (auto stateEvent = currentState().get("m.room.history_visibility"_L1)) {
-        return stateEvent->contentJson()["history_visibility"_L1].toString();
+    if (const auto stateEvent = currentState().get("m.room.history_visibility"_L1)) {
+        return stateEvent->contentPart<QString>("history_visibility"_L1);
     }
     return {};
 }
@@ -872,10 +880,11 @@ void NeoChatRoom::addParent(const QString &parentId, bool canonical, bool setPar
 
     setState("m.space.parent"_L1, parentId, QJsonObject{{"canonical"_L1, canonical}, {"via"_L1, QJsonArray{connection()->domain()}}});
 
-    if (setParentChild) {
-        if (auto parent = static_cast<NeoChatRoom *>(connection()->room(parentId))) {
-            parent->setState("m.space.child"_L1, id(), QJsonObject{{"via"_L1, QJsonArray{connection()->domain()}}});
-        }
+    if (!setParentChild) {
+        return;
+    }
+    if (auto parent = static_cast<NeoChatRoom *>(connection()->room(parentId))) {
+        parent->setState("m.space.child"_L1, id(), QJsonObject{{"via"_L1, QJsonArray{connection()->domain()}}});
     }
 }
 
@@ -926,23 +935,27 @@ void NeoChatRoom::addChild(const QString &childId, bool setChildParent, bool can
     }
     setState("m.space.child"_L1, childId, QJsonObject{{"via"_L1, QJsonArray{connection()->domain()}}, {"suggested"_L1, suggested}, {"order"_L1, order}});
 
-    if (setChildParent) {
-        if (auto child = static_cast<NeoChatRoom *>(connection()->room(childId))) {
-            if (child->canSendState("m.space.parent"_L1)) {
-                child->setState("m.space.parent"_L1, id(), QJsonObject{{"canonical"_L1, canonical}, {"via"_L1, QJsonArray{connection()->domain()}}});
+    if (!setChildParent) {
+        return;
+    }
+    if (auto child = static_cast<NeoChatRoom *>(connection()->room(childId))) {
+        if (!child->canSendState("m.space.parent"_L1)) {
+            return;
+        }
+        child->setState("m.space.parent"_L1, id(), QJsonObject{{"canonical"_L1, canonical}, {"via"_L1, QJsonArray{connection()->domain()}}});
 
-                if (canonical) {
-                    // Only one canonical parent can exist so make sure others are set to false.
-                    auto parentEvents = child->currentState().eventsOfType("m.space.parent"_L1);
-                    for (const auto &parentEvent : parentEvents) {
-                        if (parentEvent->contentPart<bool>("canonical"_L1)) {
-                            auto content = parentEvent->contentJson();
-                            content.insert("canonical"_L1, false);
-                            setState("m.space.parent"_L1, parentEvent->stateKey(), content);
-                        }
-                    }
-                }
+        if (!canonical) {
+            return;
+        }
+        // Only one canonical parent can exist so make sure others are set to false.
+        auto parentEvents = child->currentState().eventsOfType("m.space.parent"_L1);
+        for (const auto &parentEvent : parentEvents) {
+            if (!parentEvent->contentPart<bool>("canonical"_L1)) {
+                continue;
             }
+            auto content = parentEvent->contentJson();
+            content.insert("canonical"_L1, false);
+            setState("m.space.parent"_L1, parentEvent->stateKey(), content);
         }
     }
 }
@@ -957,12 +970,14 @@ void NeoChatRoom::removeChild(const QString &childId, bool unsetChildParent)
     }
     setState("m.space.child"_L1, childId, {});
 
-    if (unsetChildParent) {
-        if (auto child = static_cast<NeoChatRoom *>(connection()->room(childId))) {
-            if (child->canSendState("m.space.parent"_L1) && child->currentState().contains("m.space.parent"_L1, id())) {
-                child->setState("m.space.parent"_L1, id(), {});
-            }
+    if (!unsetChildParent) {
+        return;
+    }
+    if (auto child = static_cast<NeoChatRoom *>(connection()->room(childId))) {
+        if (!child->canSendState("m.space.parent"_L1) || !child->currentState().contains("m.space.parent"_L1, id())) {
+            return;
         }
+        child->setState("m.space.parent"_L1, id(), {});
     }
 }
 
@@ -1094,10 +1109,8 @@ void NeoChatRoom::setPushNotificationState(PushNotificationState::State state)
         const QList<PushCondition> conditions = {pushCondition};
 
         // Add new override rule and make sure it's enabled
-        auto job = connection()->callApi<SetPushRuleJob>("override"_L1, id(), actions, QString(), QString(), conditions, QString());
-        connect(job, &BaseJob::success, this, [this]() {
-            auto enableJob = connection()->callApi<SetPushRuleEnabledJob>("override"_L1, id(), true);
-            connect(enableJob, &BaseJob::success, this, [this]() {
+        connection()->callApi<SetPushRuleJob>("override"_L1, id(), actions, QString(), QString(), conditions, QString()).onResult([this]() {
+            connection()->callApi<SetPushRuleEnabledJob>("override"_L1, id(), true).onResult([this]() {
                 m_pushNotificationStateUpdating = false;
             });
         });
@@ -1120,10 +1133,8 @@ void NeoChatRoom::setPushNotificationState(PushNotificationState::State state)
         // No conditions for a room rule
         const QList<PushCondition> conditions;
 
-        auto setJob = connection()->callApi<SetPushRuleJob>("room"_L1, id(), actions, QString(), QString(), conditions, QString());
-        connect(setJob, &BaseJob::success, this, [this]() {
-            auto enableJob = connection()->callApi<SetPushRuleEnabledJob>("room"_L1, id(), true);
-            connect(enableJob, &BaseJob::success, this, [this]() {
+        connection()->callApi<SetPushRuleJob>("room"_L1, id(), actions, QString(), QString(), conditions, QString()).onResult([this]() {
+            connection()->callApi<SetPushRuleEnabledJob>("room"_L1, id(), true).onResult([this]() {
                 m_pushNotificationStateUpdating = false;
             });
         });
@@ -1151,10 +1162,8 @@ void NeoChatRoom::setPushNotificationState(PushNotificationState::State state)
         const QList<PushCondition> conditions;
 
         // Add new room rule and make sure enabled
-        auto setJob = connection()->callApi<SetPushRuleJob>("room"_L1, id(), actions, QString(), QString(), conditions, QString());
-        connect(setJob, &BaseJob::success, this, [this]() {
-            auto enableJob = connection()->callApi<SetPushRuleEnabledJob>("room"_L1, id(), true);
-            connect(enableJob, &BaseJob::success, this, [this]() {
+        connection()->callApi<SetPushRuleJob>("room"_L1, id(), actions, QString(), QString(), conditions, QString()).onResult([this]() {
+            connection()->callApi<SetPushRuleEnabledJob>("room"_L1, id(), true).onResult([this]() {
                 m_pushNotificationStateUpdating = false;
             });
         });
@@ -1164,6 +1173,22 @@ void NeoChatRoom::setPushNotificationState(PushNotificationState::State state)
     Q_EMIT pushNotificationStateChanged(m_currentPushNotificationState);
 }
 
+void NeoChatRoom::loadPinnedMessage()
+{
+    const auto events = pinnedEventIds();
+    if (!events.isEmpty()) {
+        const QString &mostRecentEventId = events.last();
+        connection()->callApi<GetOneRoomEventJob>(id(), mostRecentEventId).then([this](const auto &job) {
+            auto event = fromJson<event_ptr_tt<RoomEvent>>(job->jsonData());
+            if (auto encEv = eventCast<EncryptedEvent>(event.get())) {
+                event = decryptMessage(*encEv);
+            }
+            m_pinnedMessage = EventHandler::richBody(this, event.get());
+            Q_EMIT pinnedMessageChanged();
+        });
+    }
+}
+
 void NeoChatRoom::updatePushNotificationState(QString type)
 {
     if (type != "m.push_rules"_L1 || m_pushNotificationStateUpdating) {
@@ -1221,20 +1246,15 @@ void NeoChatRoom::updatePushNotificationState(QString type)
 
 void NeoChatRoom::reportEvent(const QString &eventId, const QString &reason)
 {
-    auto job = connection()->callApi<ReportContentJob>(id(), eventId, -50, reason);
-    connect(job, &BaseJob::finished, this, [this, job]() {
-        if (job->error() == BaseJob::Success) {
-            Q_EMIT showMessage(MessageType::Positive, i18n("Report sent successfully."));
-        }
+    auto job = connection()->callApi<ReportContentJob>(id(), eventId, -50, reason).onResult([this]() {
+        Q_EMIT showMessage(MessageType::Positive, i18n("Report sent successfully."));
     });
 }
 
 QByteArray NeoChatRoom::getEventJsonSource(const QString &eventId)
 {
-    auto evtIt = findInTimeline(eventId);
-    if (evtIt != messageEvents().rend() && is<RoomEvent>(**evtIt)) {
-        const auto event = evtIt->viewAs<RoomEvent>();
-        return QJsonDocument(event->fullJson()).toJson();
+    if (const auto evtIt = findInTimeline(eventId); evtIt != messageEvents().rend() && is<RoomEvent>(**evtIt)) {
+        return QJsonDocument(evtIt->viewAs<RoomEvent>()->fullJson()).toJson();
     }
     return {};
 }
@@ -1258,7 +1278,7 @@ void NeoChatRoom::openEventMediaExternally(const QString &eventId)
         return;
     }
     downloadFile(eventId,
-                 QUrl(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + u'/'
+                 QUrl(u"file:"_s + QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + u'/'
                       + evtIt->event()->id().replace(u':', u'_').replace(u'/', u'_').replace(u'+', u'_') + fileNameToDownload(eventId)));
     connect(
         this,
@@ -1312,6 +1332,19 @@ void NeoChatRoom::copyEventMedia(const QString &eventId)
     }
 }
 
+FileTransferInfo NeoChatRoom::cachedFileTransferInfo(const QString &eventId) const
+{
+    if (eventId.isEmpty()) {
+        return {};
+    }
+
+    const auto eventResult = getEvent(eventId);
+    if (!eventResult.first) {
+        return {};
+    }
+    return cachedFileTransferInfo(eventResult.first);
+}
+
 FileTransferInfo NeoChatRoom::cachedFileTransferInfo(const Quotient::RoomEvent *event) const
 {
     QString mxcUrl;
@@ -1370,6 +1403,20 @@ ChatBarCache *NeoChatRoom::threadCache() const
     return m_threadCache;
 }
 
+ChatBarCache *NeoChatRoom::cacheForType(ChatBarType::Type type) const
+{
+    switch (type) {
+    case ChatBarType::Room:
+        return m_mainCache;
+    case ChatBarType::Edit:
+        return m_editCache;
+    case ChatBarType::Thread:
+        return m_threadCache;
+    default:
+        return nullptr;
+    }
+}
+
 void NeoChatRoom::replyLastMessage()
 {
     const auto &timelineBottom = messageEvents().rbegin();
@@ -1495,13 +1542,11 @@ void NeoChatRoom::download(const QString &eventId, const QUrl &localFilename)
 
 void NeoChatRoom::mapAlias(const QString &alias)
 {
-    auto getLocalAliasesJob = connection()->callApi<GetLocalAliasesJob>(id());
-    connect(getLocalAliasesJob, &BaseJob::success, this, [this, getLocalAliasesJob, alias] {
-        if (getLocalAliasesJob->aliases().contains(alias)) {
+    connection()->callApi<GetLocalAliasesJob>(id()).onResult([this, alias](const auto &job) {
+        if (job->aliases().contains(alias)) {
             return;
         } else {
-            auto setRoomAliasJob = connection()->callApi<SetRoomAliasJob>(alias, id());
-            connect(setRoomAliasJob, &BaseJob::success, this, [this, alias] {
+            connection()->callApi<SetRoomAliasJob>(alias, id()).onResult([this, alias] {
                 auto newAltAliases = altAliases();
                 newAltAliases.append(alias);
                 setLocalAliases(newAltAliases);
@@ -1593,23 +1638,25 @@ void NeoChatRoom::downloadEventFromServer(const QString &eventId)
         Q_EMIT extraEventLoaded(eventId);
         return;
     }
-    auto job = connection()->callApi<GetOneRoomEventJob>(id(), eventId);
-    connect(job, &BaseJob::success, this, [this, job, eventId] {
-        // The event may have arrived in the meantime so check it's not in the timeline.
-        if (findInTimeline(eventId) != historyEdge()) {
-            Q_EMIT extraEventLoaded(eventId);
-            return;
-        }
+    connection()
+        ->callApi<GetOneRoomEventJob>(id(), eventId)
+        .then(
+            [this, eventId](const auto &job) {
+                // The event may have arrived in the meantime so check it's not in the timeline.
+                if (findInTimeline(eventId) != historyEdge()) {
+                    Q_EMIT extraEventLoaded(eventId);
+                    return;
+                }
 
-        event_ptr_tt<RoomEvent> event = fromJson<event_ptr_tt<RoomEvent>>(job->jsonData());
-        m_extraEvents.push_back(std::move(event));
-        Q_EMIT extraEventLoaded(eventId);
-    });
-    connect(job, &BaseJob::failure, this, [this, job, eventId] {
-        if (job->error() == BaseJob::NotFound) {
-            Q_EMIT extraEventNotFound(eventId);
-        }
-    });
+                event_ptr_tt<RoomEvent> event = fromJson<event_ptr_tt<RoomEvent>>(job->jsonData());
+                m_extraEvents.push_back(std::move(event));
+                Q_EMIT extraEventLoaded(eventId);
+            },
+            [this, eventId](const auto &job) {
+                if (job->error() == BaseJob::NotFound) {
+                    Q_EMIT extraEventNotFound(eventId);
+                }
+            });
 }
 
 std::pair<const Quotient::RoomEvent *, bool> NeoChatRoom::getEvent(const QString &eventId) const
@@ -1641,7 +1688,11 @@ std::pair<const Quotient::RoomEvent *, bool> NeoChatRoom::getEvent(const QString
 
 const RoomEvent *NeoChatRoom::getReplyForEvent(const RoomEvent &event) const
 {
+#if Quotient_VERSION_MINOR > 9
+    const QString &replyEventId = event.replyEventId(true);
+#else
     const QString &replyEventId = event.contentJson()["m.relates_to"_L1].toObject()["m.in_reply_to"_L1].toObject()["event_id"_L1].toString();
+#endif
     if (replyEventId.isEmpty()) {
         return {};
     };
@@ -1701,6 +1752,10 @@ void NeoChatRoom::setRoomState(const QString &type, const QString &stateKey, con
 
 NeochatRoomMember *NeoChatRoom::qmlSafeMember(const QString &memberId)
 {
+    if (memberId.isEmpty()) {
+        return nullptr;
+    }
+
     if (!m_memberObjects.contains(memberId)) {
         auto member = m_memberObjects.emplace(memberId, std::make_unique<NeochatRoomMember>(this, memberId)).first->second.get();
         QQmlEngine::setObjectOwnership(member, QQmlEngine::CppOwnership);
@@ -1736,11 +1791,7 @@ bool NeoChatRoom::eventIsThreaded(const QString &eventId) const
         return false;
     }
 
-#if Quotient_VERSION_MINOR > 9 || (Quotient_VERSION_MINOR == 9 && Quotient_VERSION_PATCH > 1)
     return event->isThreaded() || threads().contains(eventId);
-#else
-    return event->isThreaded();
-#endif
 }
 
 QString NeoChatRoom::rootIdForThread(const QString &eventId) const
@@ -1751,11 +1802,9 @@ QString NeoChatRoom::rootIdForThread(const QString &eventId) const
     }
 
     auto rootId = event->threadRootEventId();
-#if Quotient_VERSION_MINOR > 9 || (Quotient_VERSION_MINOR == 9 && Quotient_VERSION_PATCH > 1)
     if (rootId.isEmpty() && threads().contains(eventId)) {
         rootId = event->id();
     }
-#endif
     return rootId;
 }
 
@@ -1764,4 +1813,25 @@ void NeoChatRoom::setHiddenFilter(std::function<bool(const Quotient::RoomEvent *
     NeoChatRoom::m_hiddenFilter = hiddenFilter;
 }
 
+bool NeoChatRoom::roomCreatorHasUltimatePowerLevel() const
+{
+    bool ok = false;
+    auto version = this->version().toInt(&ok);
+    // This is terrible. For non-numeric room versions, I don't think there's a way of knowing whether they're pre- or post hydra.
+    // We just assume they are. Shouldn't matter for normal users anyway.
+    return !ok || version > 11;
+}
+
+bool NeoChatRoom::isCreator(const QString &userId) const
+{
+    auto createEvent = currentState().get<RoomCreateEvent>();
+    return roomCreatorHasUltimatePowerLevel() && createEvent
+        && (createEvent->senderId() == userId || createEvent->contentPart<QStringList>(u"additional_creators"_s).contains(userId));
+}
+
+QString NeoChatRoom::pinnedMessage() const
+{
+    return m_pinnedMessage;
+}
+
 #include "moc_neochatroom.cpp"
